/*
 * named constants stored in a hash
 */

#include <stdio.h>
#include <stdlib.h>

#include <hash.h>

#include "consthash.h"


extern void yyerror(const char *str);
extern int verbose;


/* the hash */
static hash consthash = NULL;


/* helper function: print (k,v) pair */
static void printkv( FILE *out, hashkey k, hashvalue v )
{
	fprintf( out, "\t  %.20s: %ld\n", k, (long)v );
}


/* helper function: free value */
static void freev( hashvalue v )
{
	/* DO NOTHING */
}


void init_consthash( int argc, char **argv )
{
	consthash = hashCreate( &printkv, &freev, NULL );
	int i;
	for( i=1; i<argc; i++ )
	{
		char key[8];
		sprintf( key, "arg%d", i );
		hashSet( consthash, key, (hashvalue)atol(argv[i]) );
	}
	long power;
	for( i=0, power=1; i<32; i++, power<<=1 )
	{
		char key[8];
		sprintf( key, "t%d", i );
		hashSet( consthash, key, (hashvalue)power );
	}
	hashSet( consthash, "howmanyroads", (hashvalue)42 );
	hashSet( consthash, "one",   (hashvalue)1 );
	hashSet( consthash, "two",   (hashvalue)2 );
	hashSet( consthash, "three", (hashvalue)3 );
	hashSet( consthash, "four",  (hashvalue)4 );
	hashSet( consthash, "five",  (hashvalue)5 );
	hashSet( consthash, "six",   (hashvalue)6 );
	hashSet( consthash, "seven", (hashvalue)7 );
	hashSet( consthash, "eight", (hashvalue)8 );
	hashSet( consthash, "nine",  (hashvalue)9 );
	hashSet( consthash, "ten",   (hashvalue)10 );
	if( verbose )
	{
		fprintf( stderr, "\n\tInitialized consthash containing:\n" );
		hashDump( stderr, consthash );
	}
}


void destroy_consthash( void )
{
	hashFree( consthash );
}


int lookup_const( char *constname )
{
	hashvalue v = NULL;
	int present = hashPresent( consthash, constname, &v );
	if( !present )
	{
		char mesg[1024];
		sprintf( mesg, "undefined constant '%s'", constname );
		yyerror( mesg );
		return -1;
	}
	return (int)(long)v;
}


void foreach_constant( constkvcb cb, void *extra )
{
	hashForeach( consthash, (hashforeachcb)cb, extra );
}
