%{
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "consthash.h"
#include "types.h"
#include "arithfuncs.h"
#include "boolfuncs.h"


extern int yyparse( void );
extern int yylex (void);
extern int yylineno;
extern BOOL verbose;


program prog_result = NULL;
int yyerrors   = 0;


void yyerror(const char *str)
{
        fprintf(stderr,"line %d: error: %s\n", yylineno, str);
	yyerrors++;
}

 
int yywrap( void )
{
        return 1;
} 
%}

%union 
{
        int n;
        char *s;
	expr e;
	bexpr b;
	guard g;
	guardlist gl;
	fdefn f;
	flist fl;
}

%token COLONCOLON IMPLIES EQ GT NE TRUEV PLUS MINUS MUL
       DIV MOD OPEN CLOSE GUARD IS INTTYPE TOKERR TOKEOF
%token <n> NUMBER
%token <s> IDENT

%type <e> factor term expr
%type <b> bexpr
%type <g> guard;
%type <gl> guardrules;
%type <f> fdefinition;
%type <fl> defns;


%start program

%%
program      :  defns expr
		{
			prog_result = program_pair( $1, $2 );
		}
             ;

defns        :  /* empty */
		{
			$$ = flist_nil();
		}
             | defns ftypedefn    /* ignore type defns */
             | defns fdefinition
		{
			$$ = flist_cons( $2, $1 );
		}
             ;

ftypedefn    : IDENT COLONCOLON type IMPLIES type
	       {
			if( verbose )
			{
				fprintf( stderr,
					"\t- type of function %s : "
					"Int->Int\n", $1 );
			}
			free_string( $1 );
	       }
	     ;

type         : INTTYPE
             ;

fdefinition : IDENT IDENT IS expr
	       {
			$$ = fdefn_onerule( $1, $2, $4 );
			if( verbose )
			{
				fprintf( stderr, "\t- simple one rule "
					"function %s( %s ) defined\n",
					$1, $2 );
			}
	       }
	     | IDENT IDENT guardrules
	       {
			guardlist rightorder = reverse_guardlist($3);
			free_guardlist_without_guard( $3 );
			$$ = fdefn_manyrules( $1, $2, rightorder );
			if( verbose )
			{
				fprintf( stderr, "\t- guarded "
				        "function %s( %s ) defined\n",
					$1, $2 );
			}
	       }
	     ;

guardrules   : guard
		{
			$$ = guardlist_cons( $1, guardlist_nil() );
		}
             | guardrules guard
		{
			$$ = guardlist_cons( $2, $1 );
		}
	     ;

guard        : GUARD bexpr IS expr
		{
			$$ = guard_pair( $2, $4 );
		}
	     ;

bexpr        : expr EQ expr	{ $$ = equals( $1, $3 ); }
	     | expr NE expr     { $$ = notequals( $1, $3 ); }
	     | expr GT expr     { $$ = greaterthan( $1, $3 ); }
	     | TRUEV            { $$ = bexpr_truev(); }
	     ;

expr         : expr PLUS term   { $$ = plus( $1, $3 ); }
             | expr MINUS term  { $$ = minus( $1, $3 ); }
	     | term
	     ;

term         : term MUL factor { $$ = times( $1, $3 ); }
             | term DIV factor { $$ = divide( $1, $3 ); }
             | term MOD factor { $$ = mod( $1, $3 ); }
	     | factor
	     ;

factor       : NUMBER		     { $$ = expr_num( $1 ); }
	     | OPEN expr CLOSE       { $$ = $2; }
	     | IDENT OPEN expr CLOSE { $$ = expr_call($1,$3); }
	     | IDENT                 { $$ = expr_id( $1 ); }
	     ;
