/*
 * function call set: a set of all functions called
 */

#include <stdio.h>
#include <stdbool.h>

#include <set.h>

#include "types.h"
#include "callset.h"


extern int yyerrors;
extern void yyerror(const char *str);
extern bool verbose;


/* the set */
static set callset = NULL;


/* helper function: print k out */
static void printkv( FILE *out, setkey k )
{
	fprintf( out, "\t  %s\n", k );
}


void init_callset( void )
{
	callset = setCreate( &printkv );
	if( verbose )
	{
		fprintf( stderr, "\tInitialized callset" );
		//setDump( stderr, callset );
		fprintf( stderr, "\n" );
	}
}


void destroy_callset( void )
{
	setFree( callset );
}


void mark_called( char *name )
{
	setAdd( callset, name );
	if( yyerrors==0 && verbose )
	{
		fprintf( stderr, "\t- marked call to function %s\n", name );
	}
}


void foreach_call( callcb cb )
{
	setForeach( callset, (setforeachcb)cb, NULL );
}


void print_callset( FILE *out )
{
	fprintf( stderr, "callset now contains: " );
	setDump( out, callset );
}
