/*
 * function definitions stored in a hash
 */

#include <stdio.h>
#include <stdbool.h>

#include <hash.h>

#include "types.h"
#include "funchash.h"


extern int yyerrors;
extern void yyerror(const char *str);
extern bool verbose;


/* the hash */
static hash funchash = NULL;


/* helper function: print (k,v) pair */
static void printkv( FILE *out, hashkey k, hashvalue v )
{
	print_fdefn( out, (fdefn)v );
	fprintf( out, "\n" );
}


/* helper function: free hashvalue (an fdefn) */
static void freev( hashvalue v )
{
	free_fdefn( (fdefn)v );
}


void init_funchash( void )
{
	funchash = hashCreate( &printkv, &freev, NULL );
	if( verbose )
	{
		fprintf( stderr, "\tInitialized funchash\n" );
		//hashDump( stderr, funchash );
	}
}


void destroy_funchash( void )
{
	hashFree( funchash );
}


bool lookup_function( char *name )
{
	hashvalue v = hashFind( funchash, name );
	return( v != NULL );
}


bool lookup_function_flagerror( char *name )
{
	hashvalue v = hashFind( funchash, name );
	if( v == NULL )
	{
		char mesg[1024];
		sprintf( mesg, "undefined function '%s'", name );
		yyerror( mesg );
		return 0;
	}
	return 1;
}


void define_function( char *name, fdefn defn )
{
	hashvalue v = hashFind( funchash, name );
	if( v == NULL )
	{
		hashSet( funchash, name, (hashvalue)defn );
		if( yyerrors==0 && verbose )
		{
			fprintf( stderr, "\t- defined function %s( int )\n",
				 name );
		}
	} else
	{
		char mesg[1024];
		sprintf( mesg, "redefined function '%s'", name );
		yyerror( mesg );
	}
}


void foreach_function( fdefcb cb )
{
	hashForeach( funchash, (hashforeachcb)cb, NULL );
}


void print_funchash( FILE *out )
{
	//fprintf( stderr, "debug: funchash:\n" );
	hashDump( out, funchash );
}
